Package: mclust
Version: 6.1.2
Date: 2025-10-30
Title: Gaussian Mixture Modelling for Model-Based Clustering,
        Classification, and Density Estimation
Description: Gaussian finite mixture models fitted via EM algorithm for
  model-based clustering, classification, and density estimation, 
  including Bayesian regularization, dimension reduction for 
  visualisation, and resampling-based inference.
Authors@R: c(person("Chris", "Fraley", role = "aut"),
             person("Adrian E.", "Raftery", role = "aut",
                    comment = c(ORCID = "0000-0002-6589-301X")),
             person("Luca", "Scrucca", role = c("aut", "cre"),
                    email = "luca.scrucca@unibo.it",
                    comment = c(ORCID = "0000-0003-3826-0484")),
             person("Thomas Brendan", "Murphy", role = "ctb",
                    comment = c(ORCID = "0000-0002-5668-7046")),
             person("Michael", "Fop", role = "ctb",
                    comment = c(ORCID = "0000-0003-3936-2757")))
Depends: R (>= 3.0)
Imports: stats, utils, graphics, grDevices
Suggests: knitr (>= 1.4), rmarkdown (>= 2.10), mix (>= 1.0), geometry
        (>= 0.4), MASS
License: GPL (>= 2)
URL: https://mclust-org.github.io/mclust/
VignetteBuilder: knitr
Repository: CRAN
ByteCompile: true
NeedsCompilation: yes
LazyData: yes
Encoding: UTF-8
Packaged: 2025-10-31 06:55:34 UTC; luca
Author: Chris Fraley [aut],
  Adrian E. Raftery [aut] (ORCID:
    <https://orcid.org/0000-0002-6589-301X>),
  Luca Scrucca [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3826-0484>),
  Thomas Brendan Murphy [ctb] (ORCID:
    <https://orcid.org/0000-0002-5668-7046>),
  Michael Fop [ctb] (ORCID: <https://orcid.org/0000-0003-3936-2757>)
Maintainer: Luca Scrucca <luca.scrucca@unibo.it>
Date/Publication: 2025-10-31 11:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 09:17:18 UTC; unix
Archs: mclust.so.dSYM
