\name{acidity}
\alias{acidity}
\docType{data}

\title{Acidity data}

\description{
Acidity index measured in a sample of 155 lakes in the Northeastern United States.
Following Crawford et al. (1992, 1994), the data are expressed as log(ANC+50), where ANC is the acidity neutralising capacity value.
The data were also used to fit mixture of gaussian distributions by Richardson and Green (1997), and by McLachlan and Peel (2000, Sec. 6.6.2).
}

\usage{data(acidity)}

\source{\code{https://www.stats.bris.ac.uk/~peter/mixdata}}

\references{
Crawford, S. L. (1994) An application of the Laplace method to finite mixture distribution. \emph{Journal of the American Statistical Association}, 89, 259--267.

Crawford, S. L., DeGroot, M. H., Kadane, J. B., and Small, M. J. (1994) Modeling lake chemistry distributions: Approximate Bayesian methods for estimating a finite mixture model. \emph{Technometrics}, 34, 441--453.

McLachlan, G. and Peel, D. (2000) \emph{Finite Mixture Models}. Wiley, New York.

Richardson, S. and Green, P. J. (1997) On Bayesian analysis of mixtures with unknown number of components (with discussion). \emph{Journal of the Royal Statistical Society, Series B}, 59, 731--792.
}

\keyword{datasets}
