\name{clustCombi-internal}
\title{Internal clustCombi functions}
\alias{combi}
\alias{pcws2_reg}
\alias{pcws3_reg}
\alias{xlog}
\description{
  Internal functions not intended to be called directly by users.
}

\keyword{internal}
